/*
 * MajakaLib.c
 *
 * Created: 2.04.2013 11:48:53
 *  Author: Rain
 */ 


#include "MajakaLib.h"

volatile uint8_t	ir_channel = 0;
volatile uint8_t	ir_kanal = 0;
volatile uint8_t	ir_interrupt_drive = 1;
volatile uint16_t	ir_newUART = 0;

static const pin led_red	= PIN(C, 5);
static const pin led_yellow	= PIN(C, 4);
static const pin led_green	= PIN(C, 3);

ISR(USART1_RX_vect)	//IR anduritelt tuli uut datat
{
	unsigned char c;
	char text[5];
	c = usart_read_char(port);
	
	lcd_gfx_goto_char_xy(0,3);
	sprintf(text,"%d",c);
	lcd_gfx_write_string(text);
	if(c == ((ir_channel<<4)|(ir_channel)))		//Kui mlemad nevad kanalit
	{
		ir_newUART = 0;
		if(ir_interrupt_drive)
		{
			pin_clear(led_yellow);
			pin_set(led_green);
			pin_set(led_red);
			
			dcmotor_drive(RIGHT_MT,RIGHT_FWD);
			dcmotor_drive(LEFT_MT,LEFT_FWD);
		}
		ir_kanal = 2;
	}
	else
	{
		if ((c & 0x0F) == ir_channel)				// Kui parem neb kanalit
		{
			ir_newUART = 0;
			if(ir_interrupt_drive)
			{
				pin_set(led_yellow);
				pin_set(led_green);
				pin_clear(led_red);
				
				dcmotor_drive(RIGHT_MT,RIGHT_FWD);
				dcmotor_drive(LEFT_MT,LEFT_REV);
			}
			ir_kanal = 1;
		}
		else if ((c & 0xF0) == (ir_channel << 4))	// Kui vasak neb kanalit
		{
			ir_newUART = 0;
			if(ir_interrupt_drive)
			{
				pin_set(led_yellow);
				pin_set(led_red);
				pin_clear(led_green);
				
				dcmotor_drive(RIGHT_MT,RIGHT_REV);
				dcmotor_drive(LEFT_MT, LEFT_FWD);
			}
			ir_kanal = 3;
		}
		else										// Kui mitte keegi ei ne kanalit
		{
			if(ir_interrupt_drive)
			{
				pin_set(led_yellow);
				pin_set(led_red);
				pin_set(led_green);
				//stop();
			}
		}
	}
}

ISR(TIMER1_OVF_vect,ISR_NOBLOCK) // Umbes iga 35.5ms
{
	//Pole uut datat RC5'st tulnud, timeout
	if (ir_newUART > 100)
	{
		ir_kanal = 0;
		ir_newUART = 101;
	}
	else
		ir_newUART++;
}

void IR_Channel(uint8_t channel, uint8_t autodrive)
{
	ir_channel = channel;
	ir_interrupt_drive = autodrive;
}

void IR_Init()
{
	//Disable watchdog and clear reset interrupts
	MCUSR &= ~(1<<WDRF);
	WDTCSR |= (1<<WDCE) | (1<<WDE);
	WDTCSR = 0x00;

	//Seadista LEDid
	pin_setup_output(led_yellow);
	pin_setup_output(led_green);
	pin_setup_output(led_red);
	pin_set(led_red);
	pin_set(led_green);
	pin_clear(led_yellow);
	
	//Kivita USART IR majakate vastu vtmiseks
	usart_init_async(port,	USART_DATABITS_8, USART_STOPBITS_ONE,
							USART_PARITY_NONE, USART_BAUDRATE_ASYNC(4800));
	UCSR1B |= (1<<RXCIE1);
	
	//Seadista mootorid
	dcmotor_init(RIGHT_MT);
	dcmotor_init(LEFT_MT);
	dcmotor_drive(LEFT_MT,0);
	dcmotor_drive(RIGHT_MT,0);

	// Taimer 1 normaalreiimi, taktijagur 8
	// Taimer 1 letitumise katkestuse lubamine

	TCCR1A	= 0;
	TCCR1B	= (1<<CS10);
	TCCR1C	= 0;
	
	TIMSK1	|= (1<<TOIE1);
	TIFR1	|= (1<<TOV1);
	sei();
}

uint8_t IR_Read()
{
	return ir_kanal;
}

